#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis
{
	public class ScalingProblem4 : ArtificialRegressionDataDescriptor
	{

		public override string Name { get { return "JKK-4 F4(X1,X2,X3,X4,X5,X6,X7,X8,X9,X10) = X1/X2 + X3/X4 + X5/X6 + X1*X7/X9 + X3*X6/X10)"; } }
		public override string Description
		{
			get
			{
				return "Function: F4(X1,X2,X3,X4,X5,X6,X7,X8,X9,X10) = X1*X2 + X3*X4 + X5*X6 + X1*X7*X9 + X3*X6*X10)" + Environment.NewLine
				+ "Data: 250 points X1, X2, X3, X4, X5, X6, X7, X8, X9, X10 = Rand(10, 20)" + Environment.NewLine
				+ "Function Set: +, -, *, /, square, e^x, e^-x, x^eps, x + eps, x * eps";
			}
		}
		protected override string TargetVariable { get { return "Y"; } }
		protected override string[] VariableNames { get { return new string[] { "X1", "X2", "X3", "X4", "X5", "X6", "X7", "X8", "X9", "X10", "Y" }; } }
		protected override string[] AllowedInputVariables { get { return new string[] { "X1", "X2", "X3", "X4", "X5", "X6", "X7", "X8", "X9", "X10" }; } }
		protected override int TrainingPartitionStart { get { return 0; } }
		protected override int TrainingPartitionEnd { get { return 250; } }
		protected override int TestPartitionStart { get { return 250; } }
		protected override int TestPartitionEnd { get { return 1000; } }

		public int Seed { get; private set; }

		public ScalingProblem4() : this((int)DateTime.Now.Ticks) { }

		public ScalingProblem4(int seed) : base()
		{
			Seed = seed;
		}
		protected override List<List<double>> GenerateValues()
		{
			var rand = new MersenneTwister((uint)Seed);
			var data = Enumerable.Range(0, AllowedInputVariables.Count()).Select(_ => Enumerable.Range(0, TestPartitionEnd).Select(__ => rand.NextDouble() * 10 + 10).ToList()).ToList();

			double x1, x2, x3, x4, x5, x6, x7, x8, x9, x10;

			List<double> results = new List<double>();
			for (int i = 0; i < data[0].Count; i++)
			{
				x1 = data[0][i];
				x2 = data[1][i];
				x3 = data[2][i];
				x4 = data[3][i];
				x5 = data[4][i];
				x6 = data[5][i];
				x7 = data[6][i];
				x8 = data[7][i];
				x9 = data[8][i];
				x10 = data[9][i];
				results.Add(x1 / x2 + x3 / x4 + x5 / x6 + x1 * x7 / x9 + x3 * x6 / x10);
			}
			data.Add(results);

			return data;
		}
	}
}
